// notenrechner
// Copyright (C) 2025 Frieder Hannenheim
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

import studiengaenge from '../build/studiengaenge.json'

var select = document.getElementById("studiengaenge") as HTMLSelectElement;


for (let key in studiengaenge) {
    let studiengang = studiengaenge[key as keyof typeof studiengaenge];
    let option = document.createElement('option');
    option.value = key;
    option.text = String(studiengang.display_name);

    let optGroupId = "optgroup" + studiengang.type.toLowerCase();
    let optionGroup = document.getElementById(optGroupId) as HTMLOptGroupElement;
    if (optionGroup == null) {
        optionGroup = document.createElement("optgroup");
        optionGroup.id = optGroupId;
        optionGroup.label = studiengang.type;
        select.add(optionGroup);
    }

    optionGroup.appendChild(option);
}
select.selectedIndex = 0;

var noten = document.getElementById("noten") as HTMLFormElement;
var notenInner = document.getElementById("noten") as HTMLDivElement;

select.onchange = function changed() {
    notenInner.innerHTML = "";
    let gewichtung_index = select.selectedOptions[0].value as keyof typeof studiengaenge;
    let gewichtung = studiengaenge[gewichtung_index];

    let table = document.createElement('table');

    for (let modulIndex in gewichtung.module) {
        let modul = gewichtung.module[modulIndex];
        let label = document.createElement('label');
        let input = document.createElement('input');
        let row = table.insertRow();


        input.id = modul.short + "input";
        input.type = "number";
        input.required = true;
        input.min = "1";
        input.max = "5";
        input.step = "0.1";
        input.addEventListener("input", () => {
            // Validate with the built-in constraints
            input.setCustomValidity("");
            if (!input.validity.valid) {
                return;
            }

            let note = parseFloat(input.value.replace(",", "."));
            if (note > 4) {
                input.setCustomValidity("Keine Berechnung möglich bei nicht bestandenen Prüfungen (mit > 4.0)")
            }
        });
        input.className = "noteninput";

        label.htmlFor = input.id;
        label.textContent = modul.name;

        row.insertCell().appendChild(label);
        let input_cell = row.insertCell();
        input_cell.className = "noteninputcell";
        if (modul.lp != 0) {
            input_cell.appendChild(input);
        } else {
            let hinweis = document.createElement('small');
            hinweis.innerText = "Keine Note.\n Muss bestanden sein."
            input_cell.appendChild(hinweis);
        }
    }

    let submit = document.createElement('input');
    submit.type = "submit";
    submit.value = "Berechnen";
    let row = table.insertRow();
    row.insertCell();
    let cell = row.insertCell();
    cell.className = "noteninputcell";
    cell.appendChild(submit);

    notenInner.appendChild(table);


    let result = document.createElement('p');

    notenInner.appendChild(result);

    noten.onsubmit = function calculate(event) {
        event.preventDefault();
        let sum = 0;
        let credits = 0;

        for (let modulIndex in gewichtung.module) {
            let modul = gewichtung.module[modulIndex];
            if (modul.lp == 0) {
                continue;
            }

            let noteInput = document.getElementById(modul.short + "input") as HTMLInputElement;
            let noteString = noteInput.value;
            let note = parseFloat(noteString.replace(",", "."));

            sum += modul.lp * note;
            credits += modul.lp;
        }

        let endnote = Math.floor((sum / credits) * 10) / 10;

        result.innerText = "Deine Note: " + String(endnote);
    }
}
